@@ -38,11 +38,11 @@
  * 3.1.2b1 would be 0x03010201.  Snapshot from Jan 2, 2003 would be
  * 0xFF030102. 
  * See TSK_VERSION_STR for string form. */ 
-#define TSK_VERSION_NUM 0x030103ff
+#define TSK_VERSION_NUM 0x030201ff
 
 /** Version of code in string form. See TSK_VERSION_NUM for 
  * integer form. */
-#define TSK_VERSION_STR "3.1.3"
+#define TSK_VERSION_STR "3.2.1"
 
 
 /* include the TSK-specific header file that we created in autoconf
@@ -67,7 +67,8 @@ extern "C" {
     typedef enum {
         TSK_OK,                 ///< Ok -- success
         TSK_ERR,                ///< System error -- should abort
-        TSK_COR                 ///< Data is corrupt, can still process another set of data
+        TSK_COR,                 ///< Data is corrupt, can still process another set of data
+        TSK_STOP                ///< Stop further processing, not an error though. 
     } TSK_RETVAL_ENUM;
 
 
@@ -238,6 +239,7 @@ extern "C" {
 #define TSK_ERR_VS	0x04000000
 #define TSK_ERR_FS	0x08000000
 #define TSK_ERR_HDB	0x10000000
+#define TSK_ERR_AUTO 0x20000000
 #define TSK_ERR_MASK	0x00ffffff
 
 #define TSK_ERR_AUX_MALLOC	(TSK_ERR_AUX | 0)
@@ -304,6 +306,11 @@ extern "C" {
 #define TSK_ERR_HDB_CORRUPT     (TSK_ERR_HDB | 11)
 #define TSK_ERR_HDB_MAX		12
 
+#define TSK_ERR_AUTO_DB (TSK_ERR_AUTO | 0)
+#define TSK_ERR_AUTO_CORRUPT (TSK_ERR_AUTO | 1)
+#define TSK_ERR_AUTO_UNICODE (TSK_ERR_AUTO | 2)
+#define TSK_ERR_AUTO_NOTOPEN (TSK_ERR_AUTO | 3)
+#define TSK_ERR_AUTO_MAX 4
 //@}
 
 
