@@ -78,7 +78,7 @@ find_parent_act(TSK_FS_FILE * fs_file, const char *a_path, void *ptr)
 }
 
 
-static uint8_t
+uint8_t
 fatfs_dir_buf_add(FATFS_INFO * fatfs, TSK_INUM_T par_inum,
     TSK_INUM_T dir_inum)
 {
@@ -190,10 +190,9 @@ fatfs_dent_parse_buf(FATFS_INFO * fatfs, TSK_FS_DIR * a_fs_dir, char *buf,
         /* cycle through the directory entries */
         for (idx = 0; idx < fatfs->dentry_cnt_se; idx++, dep++) {
             fatfs_dentry *dir;
-            int i;
 
             /* is it a valid dentry? */
-            if (0 == fatfs_isdentry(fatfs, dep)) {
+            if (0 == fatfs_isdentry(fatfs, dep, 1)) {
                 if (tsk_verbose)
                     tsk_fprintf(stderr,
                         "fatfs_dent_parse_buf: Entry %u is invalid\n",
@@ -294,8 +293,8 @@ fatfs_dent_parse_buf(FATFS_INFO * fatfs, TSK_FS_DIR * a_fs_dir, char *buf,
 
                     /* Convert the UTF16 to UTF8 */
                     UTF16 *name16 =
-                        (UTF16 *) ((uintptr_t) & lfninfo.
-                        name[lfninfo.start + 1]);
+                        (UTF16 *) ((uintptr_t) & lfninfo.name[lfninfo.
+                            start + 1]);
                     UTF8 *name8 = (UTF8 *) fs_name->name;
 
                     retVal =
@@ -322,14 +321,6 @@ fatfs_dent_parse_buf(FATFS_INFO * fatfs, TSK_FS_DIR * a_fs_dir, char *buf,
                     else
                         *name8 = '\0';
 
-                    /* Clean up name */
-                    i = 0;
-                    while (fs_name->name[i] != '\0') {
-                        if (TSK_IS_CNTRL(fs_name->name[i]))
-                            fs_name->name[i] = '^';
-                        i++;
-                    }
-
                     lfninfo.start = FATFS_MAXNAMLEN_UTF8 - 1;
                     name_ptr = fs_name->shrt_name;      // put 8.3 into shrt_name
                 }
@@ -378,14 +369,6 @@ fatfs_dent_parse_buf(FATFS_INFO * fatfs, TSK_FS_DIR * a_fs_dir, char *buf,
                 name_ptr[a] = '\0';
             }
 
-            /* Clean up name to remove control chars */
-            i = 0;
-            while (fs_name->name[i] != '\0') {
-                if (TSK_IS_CNTRL(fs_name->name[i]))
-                    fs_name->name[i] = '^';
-                i++;
-            }
-
             /* file type: FAT only knows DIR and FILE */
             if ((dir->attrib & FATFS_ATTR_DIRECTORY) ==
                 FATFS_ATTR_DIRECTORY)
@@ -623,31 +606,16 @@ fatfs_dir_open_meta(TSK_FS_INFO * a_fs, TSK_FS_DIR ** a_fs_dir,
         tsk_fs_dir_reset(fs_dir);
     }
     else {
-        if ((*a_fs_dir = fs_dir = tsk_fs_dir_alloc(a_fs, 128)) == NULL) {
+        if ((*a_fs_dir = fs_dir = tsk_fs_dir_alloc(a_fs, a_addr, 128)) == NULL) {
             return TSK_ERR;
         }
     }
 
     //  handle the orphan directory if its contents were requested
     if (a_addr == TSK_FS_ORPHANDIR_INUM(a_fs)) {
-        size_t i;
-        TSK_RETVAL_ENUM retval2 = tsk_fs_dir_find_orphans(a_fs, fs_dir);
-
-        if (retval2 == TSK_ERR)
-            return retval2;
-
-        // update the directory to parent mapping (since we bypassed the code that does this in parse)
-        for (i = 0; i < fs_dir->names_used; i++) {
-            if (fs_dir->names[i].type == TSK_FS_NAME_TYPE_DIR) {
-                if (fatfs_dir_buf_add(fatfs, a_addr,
-                        fs_dir->names[i].meta_addr))
-                    return TSK_ERR;
-            }
-        }
-        return retval2;
+        return tsk_fs_dir_find_orphans(a_fs, fs_dir);
     }
 
-
     fs_dir->fs_file = tsk_fs_file_open_meta(a_fs, NULL, a_addr);
     if (fs_dir->fs_file == NULL) {
         tsk_error_reset();
