@@ -15,15 +15,17 @@
  */
 
 #include "tsk_fs_i.h"
+#include "tsk_fatfs.h"
 
 /** \internal
-* Allocate a FS_DIR structure to load names into
+* Allocate a FS_DIR structure to load names into.  
 * 
+* @param a_addr Address of this directory. 
 * @param a_cnt target number of FS_DENT entries to fit in
 * @returns NULL on error
 */
 TSK_FS_DIR *
-tsk_fs_dir_alloc(TSK_FS_INFO * a_fs, size_t a_cnt)
+tsk_fs_dir_alloc(TSK_FS_INFO * a_fs, TSK_INUM_T a_addr, size_t a_cnt)
 {
     TSK_FS_DIR *fs_dir;
     size_t i;
@@ -42,6 +44,7 @@ tsk_fs_dir_alloc(TSK_FS_INFO * a_fs, size_t a_cnt)
         return NULL;
     }
     fs_dir->fs_info = a_fs;
+    fs_dir->addr = a_addr;
     fs_dir->tag = TSK_FS_DIR_TAG;
     for (i = 0; i < a_cnt; i++) {
         fs_dir->names[i].tag = TSK_FS_NAME_TAG;
@@ -99,6 +102,7 @@ tsk_fs_dir_reset(TSK_FS_DIR * a_fs_dir)
         a_fs_dir->fs_file = NULL;
     }
     a_fs_dir->names_used = 0;
+    a_fs_dir->addr = 0;
 }
 
 
@@ -129,6 +133,7 @@ tsk_fs_dir_copy(const TSK_FS_DIR * a_src_dir, TSK_FS_DIR * a_dst_dir)
     }
 
     a_dst_dir->names_used = a_src_dir->names_used;
+    a_dst_dir->addr = a_src_dir->addr;
     return 0;
 }
 
@@ -199,6 +204,10 @@ tsk_fs_dir_add(TSK_FS_DIR * a_fs_dir, const TSK_FS_NAME * a_fs_name)
     if (tsk_fs_name_copy(fs_name_dest, a_fs_name))
         return 1;
 
+    // add the parent address 
+    if (a_fs_dir->addr)
+        fs_name_dest->par_addr = a_fs_dir->addr;
+
     return 0;
 }
 
@@ -704,8 +713,6 @@ uint8_t
 tsk_fs_dir_make_orphan_dir_meta(TSK_FS_INFO * a_fs,
     TSK_FS_META * a_fs_meta)
 {
-    TSK_DADDR_T *addr_ptr;
-
     a_fs_meta->type = TSK_FS_META_TYPE_DIR;
     a_fs_meta->mode = 0;
     a_fs_meta->nlink = 1;
@@ -733,7 +740,7 @@ tsk_fs_dir_make_orphan_dir_meta(TSK_FS_INFO * a_fs,
     strncpy(a_fs_meta->name2->name, "$OrphanFiles",
         TSK_FS_META_NAME_LIST_NSIZE);
     if (a_fs_meta->content_len) {
-        addr_ptr = (TSK_DADDR_T *) a_fs_meta->content_ptr;
+        TSK_DADDR_T *addr_ptr = (TSK_DADDR_T *) a_fs_meta->content_ptr;
         addr_ptr[0] = 0;
     }
     a_fs_meta->size = 0;
@@ -799,9 +806,20 @@ load_orphan_dir_walk_cb(TSK_FS_FILE * a_fs_file, const char *a_path,
     FIND_ORPHAN_DATA *data = (FIND_ORPHAN_DATA *) a_ptr;
 
     // add this entry to the orphan list
-    if (a_fs_file->meta)
+    if (a_fs_file->meta) {
         tsk_list_add(&data->orphan_subdir_list, a_fs_file->meta->addr);
 
+        /* FAT file systems spend a lot of time hunting for parent 
+         * directory addresses, so we put this code in here to save
+         * the info when we have it. */
+        if ((a_fs_file->meta->type == TSK_FS_META_TYPE_DIR)
+            && (TSK_FS_TYPE_ISFAT(a_fs_file->fs_info->ftype))) {
+            if (fatfs_dir_buf_add((FATFS_INFO *) a_fs_file->fs_info,
+                    a_fs_file->name->par_addr, a_fs_file->meta->addr))
+                return TSK_WALK_ERROR;
+        }
+    }
+
     return TSK_WALK_CONT;
 }
 
@@ -841,6 +859,15 @@ find_orphan_meta_walk_cb(TSK_FS_FILE * a_fs_file, void *a_ptr)
     if (tsk_fs_dir_add(data->fs_dir, data->fs_name))
         return TSK_WALK_ERROR;
 
+    /* FAT file systems spend a lot of time hunting for parent 
+     * directory addresses, so we put this code in here to save
+     * the info when we have it. */
+    if (TSK_FS_TYPE_ISFAT(fs->ftype)) {
+        if (fatfs_dir_buf_add((FATFS_INFO *) fs,
+                TSK_FS_ORPHANDIR_INUM(fs), a_fs_file->meta->addr))
+            return TSK_WALK_ERROR;
+    }
+
     /* Go into directories to mark their contents as "seen" */
     if (a_fs_file->meta->type == TSK_FS_META_TYPE_DIR) {
 
@@ -931,7 +958,7 @@ tsk_fs_dir_find_orphans(TSK_FS_INFO * a_fs, TSK_FS_DIR * a_fs_dir)
 
     // make copy of this so that we don't need to do it again. 
     if ((a_fs->orphan_dir =
-            tsk_fs_dir_alloc(a_fs, a_fs_dir->names_used)) == NULL) {
+            tsk_fs_dir_alloc(a_fs, a_fs_dir->addr, a_fs_dir->names_used)) == NULL) {
         a_fs->isOrphanHunting = 0;
         return TSK_ERR;
     }
