@@ -77,6 +77,7 @@ extern "C" {
     * Flags that are used to specify which blocks to call the tsk_fs_block_walk() callback function with.
      */
     enum TSK_FS_BLOCK_WALK_FLAG_ENUM {
+        TSK_FS_BLOCK_WALK_FLAG_NONE = 0x00,     ///< No Flags
         TSK_FS_BLOCK_WALK_FLAG_ALLOC = 0x01,    ///< Allocated blocks
         TSK_FS_BLOCK_WALK_FLAG_UNALLOC = 0x02,  ///< Unallocated blocks
         TSK_FS_BLOCK_WALK_FLAG_CONT = 0x04,     ///< Blocks that could store file content
@@ -133,6 +134,7 @@ extern "C" {
      * Flags used for a TSK_FS_ATTR_RUN entry. 
      */
     typedef enum {
+        TSK_FS_ATTR_RUN_FLAG_NONE = 0x00,       ///< No Flag
         TSK_FS_ATTR_RUN_FLAG_FILLER = 0x01,     ///< Entry is a filler for a run that has not been seen yet in the processing (or has been lost)
         TSK_FS_ATTR_RUN_FLAG_SPARSE = 0x02      ///< Entry is a sparse run where all data in the run is zeros
     } TSK_FS_ATTR_RUN_FLAG_ENUM;
@@ -161,6 +163,7 @@ extern "C" {
      * store file content metadata. 
      */
     typedef enum {
+        TSK_FS_ATTR_FLAG_NONE = 0x00,   ///< No Flag
         TSK_FS_ATTR_INUSE = 0x01,       ///< data structure is in use
         TSK_FS_ATTR_NONRES = 0x02,      ///< Contains non-resident data (i.e. located in blocks)
         TSK_FS_ATTR_RES = 0x04, ///< Contains resident data (i.e. in a small buffer)
@@ -190,6 +193,7 @@ extern "C" {
      * Flags used by tsk_fs_file_walk to determine when the callback function should
      * be used. */
     typedef enum {
+        TSK_FS_FILE_WALK_FLAG_NONE = 0x00,      ///< No Flag
         TSK_FS_FILE_WALK_FLAG_SLACK = 0x01,     ///< Include the file's slack space in the callback.
         TSK_FS_FILE_WALK_FLAG_NOID = 0x02,      ///< Ignore the Id argument given in the API (use only the type)
         TSK_FS_FILE_WALK_FLAG_AONLY = 0x04,     ///< Provide callback with only addresses and no file content.
@@ -249,7 +253,7 @@ extern "C" {
         TSK_FS_ATTR *next;      ///< Pointer to next attribute in list
         TSK_FS_FILE *fs_file;   ///< Pointer to the file that this is from
         TSK_FS_ATTR_FLAG_ENUM flags;    ///< Flags for attribute
-        char *name;             ///< Name of attribute (could be NULL) (in UTF-8)
+        char *name;             ///< Name of attribute (in UTF-8).  Will be NULL if attribute doesn't have a name. 
         size_t name_size;       ///< Number of bytes allocated to name
         TSK_FS_ATTR_TYPE_ENUM type;     ///< Type of attribute
         uint16_t id;            ///< Id of attribute
@@ -463,6 +467,13 @@ extern "C" {
  * name is known, but the parent is not */
 #define TSK_FS_ORPHAN_STR "-ORPHAN_FILE-"
 
+    /* we are using the last inode as the special inode for the orphan directory.  Note that this
+     * macro is defined to abstract this convention, but there are many places in the code where
+     * there is implied logic about this convention. For example, inode_walks will stop before
+     * this value so that special handling can occur. */
+#define TSK_FS_ORPHANDIR_INUM(fs_info) \
+(fs_info->last_inum)
+
 
     /** 
         * inode walk callback function definition.  This is called for every file
@@ -539,6 +550,7 @@ extern "C" {
 
         TSK_INUM_T meta_addr;   ///< Address of the metadata structure that the name points to. 
         uint32_t meta_seq;      ///< Sequence number for metadata structure (NTFS only) 
+        TSK_INUM_T par_addr;    ///< Metadata address of parent directory (equal to meta_addr if this entry is for root directory). 
 
         TSK_FS_NAME_TYPE_ENUM type;     ///< File type information (directory, file, etc.)
         TSK_FS_NAME_FLAG_ENUM flags;    ///< Flags that describe allocation status etc. 
@@ -570,6 +582,8 @@ extern "C" {
         size_t names_used;      ///< Number of name structures in queue being used
         size_t names_alloc;     ///< Number of name structures that were allocated
 
+        TSK_INUM_T addr;    ///< Metadata address of this directory 
+
         TSK_FS_INFO *fs_info;   ///< Pointer to file system the directory is located in
     } TSK_FS_DIR;
 
@@ -578,6 +592,7 @@ extern "C" {
      * which files to call the callback function on. 
      */
     typedef enum {
+        TSK_FS_DIR_WALK_FLAG_NONE = 0x00,       ///< No Flags
         TSK_FS_DIR_WALK_FLAG_ALLOC = 0x01,      ///< Return allocated names in callback
         TSK_FS_DIR_WALK_FLAG_UNALLOC = 0x02,    ///< Return unallocated names in callback
         TSK_FS_DIR_WALK_FLAG_RECURSE = 0x04,    ///< Recurse into sub-directories 
@@ -626,6 +641,7 @@ extern "C" {
     /**
      * Flags used by tsk_fs_file_read */
     typedef enum {
+        TSK_FS_FILE_READ_FLAG_NONE = 0x00,      ///< No Flags
         TSK_FS_FILE_READ_FLAG_SLACK = 0x01,     ///< Allow read access into slack space
         TSK_FS_FILE_READ_FLAG_NOID = 0x02,      ///< Ignore the Id argument given in the API (use only the type)
     } TSK_FS_FILE_READ_FLAG_ENUM;
@@ -781,6 +797,7 @@ extern "C" {
      * Flags for the FS_INFO structure 
      */
     enum TSK_FS_INFO_FLAG_ENUM {
+        TSK_FS_INFO_FLAG_NONE = 0x00,   ///< No Flags
         TSK_FS_INFO_FLAG_HAVE_SEQ = 0x01        ///< File system has sequence numbers in the inode addresses.
     };
     typedef enum TSK_FS_INFO_FLAG_ENUM TSK_FS_INFO_FLAG_ENUM;
@@ -830,7 +847,7 @@ extern "C" {
                      * or when a full name_walk is performed. 
 					 */
 
-        TSK_FS_DIR *orphan_dir;         ///< Files and dirs in the top level of the $OrphanFiles directory.  NULL if orphans have not been hunted for yet. 
+        TSK_FS_DIR *orphan_dir; ///< Files and dirs in the top level of the $OrphanFiles directory.  NULL if orphans have not been hunted for yet. 
         uint8_t isOrphanHunting;        ///< Set to 1 if TSK is currently looking for Orphan files
 
          uint8_t(*block_walk) (TSK_FS_INFO * fs, TSK_DADDR_T start, TSK_DADDR_T end, TSK_FS_BLOCK_WALK_FLAG_ENUM flags, TSK_FS_BLOCK_WALK_CB cb, void *ptr);    ///< FS-specific function: Call tsk_fs_block_walk() instead. 
@@ -912,6 +929,7 @@ extern "C" {
 
 
     enum TSK_FS_BLKCAT_FLAG_ENUM {
+        TSK_FS_BLKCAT_NONE = 0x00,
         TSK_FS_BLKCAT_HEX = 0x01,
         TSK_FS_BLKCAT_ASCII = 0x02,
         TSK_FS_BLKCAT_HTML = 0x04,
@@ -924,6 +942,7 @@ extern "C" {
 
 
     enum TSK_FS_BLKLS_FLAG_ENUM {
+        TSK_FS_BLKLS_NONE = 0x00,
         TSK_FS_BLKLS_CAT = 0x01,
         TSK_FS_BLKLS_LIST = 0x02,
         TSK_FS_BLKLS_SLACK = 0x04,
@@ -947,6 +966,7 @@ extern "C" {
 
 
     enum TSK_FS_FLS_FLAG_ENUM {
+        TSK_FS_FLS_NONE = 0x00,
         TSK_FS_FLS_DOT = 0x01,
         TSK_FS_FLS_LONG = 0x02,
         TSK_FS_FLS_FILE = 0x04,
@@ -957,7 +977,7 @@ extern "C" {
     typedef enum TSK_FS_FLS_FLAG_ENUM TSK_FS_FLS_FLAG_ENUM;
     extern uint8_t tsk_fs_fls(TSK_FS_INFO * fs,
         TSK_FS_FLS_FLAG_ENUM lclflags, TSK_INUM_T inode,
-        TSK_FS_NAME_FLAG_ENUM flags, TSK_TCHAR * pre, int32_t skew);
+        TSK_FS_DIR_WALK_FLAG_ENUM flags, TSK_TCHAR * pre, int32_t skew);
 
     extern uint8_t tsk_fs_icat(TSK_FS_INFO * fs,
         TSK_INUM_T inum,
@@ -966,6 +986,7 @@ extern "C" {
 
 
     enum TSK_FS_IFIND_FLAG_ENUM {
+        TSK_FS_IFIND_NONE = 0x00,
         TSK_FS_IFIND_ALL = 0x01,
         TSK_FS_IFIND_PAR_LONG = 0x02,
     };
@@ -979,6 +1000,7 @@ extern "C" {
 
 
     enum TSK_FS_ILS_FLAG_ENUM {
+        TSK_FS_ILS_NONE = 0x00,
         TSK_FS_ILS_OPEN = 0x01,
         TSK_FS_ILS_MAC = 0x02,
         TSK_FS_ILS_LINK = 0x04,
